import os
import numpy as np
import matplotlib.pyplot as plt
from itertools import cycle
from sklearn.metrics import roc_curve, auc
class_name=["PAD", "NNW", "THW-ADR"]
n_classes = len(class_name)
if __name__ == '__main__':
    pred_grid = []
    for file in os.listdir("torch"):
        path = "pred_word_pair_label_for_each_batch/" + file
        pred_grid_batch = np.load(path)
        pred_grid.append(np.array(pred_grid_batch))

    true_grid = []
    for file in os.listdir("data/true_word_pair_label_for_each_batch"):
        path = "true_word_pair_label_for_each_batch/" + file
        data = np.load(path).tolist()
        true_grid += [c for a in data for b in a for c in b]

    n = len(pred_grid)

    pred_1 = [] # class 1
    pred_2 = [] # class 2
    pred_3 = [] # class 3
    for i in range(n):
        p = pred_grid[i]
        p_class = [p[:, :, :, i] for i in range(p.shape[3])]
        pred_1  += [c for a in p_class[0] for b in a for c in b]
        pred_2 += [c for a in p_class[1] for b in a for c in b]
        pred_3 += [c for a in p_class[2] for b in a for c in b]

    pred_class = [pred_1, pred_2, pred_3]

    pred_class = np.array(pred_class).transpose()
    true_class = np.array([true_grid, true_grid, true_grid]).transpose()
    for i in range(true_class.shape[0]):
        for j in range(true_class.shape[1]):
            if true_class[i][j] == j:
                true_class[i][j] = 1
            else:
                true_class[i][j] = 0

    # 计算每一类的ROC[ours]
    fpr = dict()
    tpr = dict()
    roc_auc = dict()
    for i in range(n_classes):
        fpr[i], tpr[i], _ = roc_curve(true_class[:, i], pred_class[:, i])
        roc_auc[i] = auc(fpr[i], tpr[i])
    lw = 2.5
    colors = ['#3682be', '#5f6694', '#df3881']
    linestyles = [".", "-", "--"]
    # colors = cycle(['C0', 'C1', 'C2', "C3", "C4", "C5"])
    for i, color in zip(range(n_classes), colors):
        if class_name[i] in ["PAD"]: continue
        plt.plot(fpr[i], tpr[i], color=color, linestyle=linestyles[i], lw=lw,
                 label='ours-token label:{0} (area = {1:0.2f})'
                       ''.format(class_name[i], roc_auc[i]))


    plt.plot([0, 1], [0, 1], 'k--', lw=lw)
    plt.xlim([0.0, 1.0])
    plt.ylim([0.0, 1.05])
    plt.xlabel('False Positive Rate')
    plt.ylabel('True Positive Rate')
    plt.title('CADEC token level multi-class ROC curve')
    plt.legend(loc="lower right")
    plt.savefig("roc_cadec.png")
    plt.show()





